// GuiButton.cpp : implementation file
//

#include "stdafx.h"
#include "GuiButton.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGuiButton


/////////////////////////////////////////////////////////////////////////
/// <b>Function: CGuiButton</b>
///
///	\param	NONE
///
/// \remarks	Constructor
///
/////////////////////////////////////////////////////////////////////////
CGuiButton::CGuiButton()
{
}


/////////////////////////////////////////////////////////////////////////
/// <b>Function: ~CGuiButton</b>
///
///	\param	NONE
///
/// \remarks	Destructor
///
/////////////////////////////////////////////////////////////////////////
CGuiButton::~CGuiButton()
{
}


BEGIN_MESSAGE_MAP(CGuiButton, CButton)
	//{{AFX_MSG_MAP(CGuiButton)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		ON_WM_ERASEBKGND()
		ON_WM_KILLFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGuiButton message handlers

/////////////////////////////////////////////////////////////////////////
/// <b>Function: SetSkin</b>
///
/// \param	uiNormal	(in) Normal Bitmap ID
///
/// \param	uiDisabled	(in) Disabled Bitmap ID
///
/// \param	clrTextColor	(in) Text Color where default color is white
///
/// \return	void 
///
/// \remarks	Sets the skin to the button
///
/////////////////////////////////////////////////////////////////////////
void CGuiButton::SetSkin(UINT uiNormal, UINT uiDisabled, COLORREF clrTextColor)
{
	TRACE(TEXT("CGuiButton::SetSkin\n"));
	// --------------------------------------
	// Free previous allocated bitmaps
	// --------------------------------------
	m_NormalBitmapDC.DeleteObject();
	m_DisabledBitmapDC.DeleteObject();

	// --------------------------------------
	// Load bitmaps corresponding to states
	// --------------------------------------
	if (uiNormal>0) 
		m_NormalBitmapDC.LoadBitmap(uiNormal);
	if (uiDisabled>0) 
		m_DisabledBitmapDC.LoadBitmap(uiDisabled);

	m_TextColor = clrTextColor;
}


/////////////////////////////////////////////////////////////////////////
/// <b>Function: PreSubclassWindow</b>
///
///	\param	NONE
///
/// \return	void 
///
/// \remarks	PreSubclassWindow to make the button is Owner Draw.
///
/////////////////////////////////////////////////////////////////////////
void CGuiButton::PreSubclassWindow() 
{
	CButton::PreSubclassWindow();

	// Modifying the style to OwnerDraw
	ModifyStyle(0, BS_OWNERDRAW );

}


/////////////////////////////////////////////////////////////////////////
/// <b>Function: OnEraseBkgnd</b>
///
/// \param	pDC	(in\out)
///
/// \return	BOOL 
///
/// \remarks	We do not want the base class to erase the background.
///
/////////////////////////////////////////////////////////////////////////
BOOL CGuiButton::OnEraseBkgnd(CDC* pDC) 
{
	// Do not erase background to be transparent
	return true; 
}


/////////////////////////////////////////////////////////////////////////
/// <b>Function: OnKillFocus</b>
///
/// \param	pNewWnd	(in\out)
///
/// \return	void 
///
/// \remarks	Validates the button so that the Focus rect is removed 
///             correctly
///
/////////////////////////////////////////////////////////////////////////
void CGuiButton::OnKillFocus(CWnd* pNewWnd)
{
	this->Invalidate();
}

/////////////////////////////////////////////////////////////////////////
/// <b>Function: DrawItem</b>
///
/// \param	lpDrawItemStruct	(in\out)
///
/// \return	void 
///
/// \remarks	Draws the button in 3 steps
///             1. Bitmap     - Draws the loaded bitmap
///             2. Text       - Draws the button text
///             3. Focus Rect - Draws the focus rect of the button
///
/////////////////////////////////////////////////////////////////////////
void CGuiButton::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
    ASSERT (lpDrawItemStruct);

	CDC btnDC ;
	btnDC.Attach(lpDrawItemStruct->hDC);	// get device context

	CBitmap* pbitmapDC=NULL;
	BITMAP bmpStruct;

	CRect btnRect;
	GetClientRect(&btnRect);

	// --------------------------------------
	// Draw the bitmap
	// --------------------------------------
	
	// Get the correct Bitmap
	if(! IsWindowEnabled())
		pbitmapDC=&m_DisabledBitmapDC;
	else
		pbitmapDC=&m_NormalBitmapDC;
	
	// Select the bitmap into a Compatible DC
	CDC *bmpDC = new CDC();
	bmpDC->CreateCompatibleDC(&btnDC);
	bmpDC->SelectObject(pbitmapDC);

	if(pbitmapDC != NULL)
	{
		pbitmapDC->GetBitmap(&bmpStruct);
		const int iDiff = 1;
		// Draw the bitmap on the button leaving one pixel from each side of the button
		btnDC.StretchBlt(iDiff, iDiff, btnRect.Width()-(2*iDiff), btnRect.Height()-(2*iDiff),bmpDC,0,0,bmpStruct.bmWidth,bmpStruct.bmHeight, SRCCOPY );
	}

	// --------------------------------------
	// Draw the Text
	// --------------------------------------

	CString sCaption;
	GetWindowText(sCaption);	// get button text

	int iLength = sCaption.GetLength();

	CRect textRect;
	textRect = lpDrawItemStruct->rcItem;

	int       oldMode = btnDC.SetBkMode(TRANSPARENT);
	COLORREF oldColor = btnDC.SetTextColor(m_TextColor);

	CSize sz;
	sz = btnDC.GetTextExtent(sCaption);
 
	BOOL bNoOfLines = FALSE; 
	UINT uiDrawTextFormat = DT_CENTER|DT_VCENTER|DT_SINGLELINE;
	
	int iOffset = btnDC.DrawText(sCaption,textRect,uiDrawTextFormat);
	
	
	// --------------------------------------
	// Draw the Focus Rect
	// --------------------------------------
	if( (iLength)  && (lpDrawItemStruct->itemState & ODS_FOCUS) )
	{
		CRect focusRect( btnRect );
		focusRect.InflateRect(-1,-1,-1,-1);
		btnDC.DrawFocusRect(&focusRect);
	}

	btnDC.SetTextColor(oldColor) ;
	btnDC.SetBkMode(oldMode) ;
	btnDC.Detach() ;	
	
}
